<?php
/**
 * Cette classe fournit une interface pour l'envoi de notifications aux membres
 * @copyright   &copy; 2005-2019 PHPBoost
 * @license     https://www.gnu.org/licenses/gpl-3.0.html GNU/GPL-3.0
 * @author      Kévin BRISSEZ <brissez.kevin@gmail.com>
 * @version     PHPBoost 5.0 - last update: 2019 11 01
 * @since       PHPBoost 5.0 - 2019 07 01
 */
class AdminNotificationsFormController extends ModuleController
  {
    
    private $form;
    
    /**
     * @var FormButtonSubmit
     */
    private $submit_button;
    
    private $lang;
    
    /**
     * @var NotificationsConfig
     */
    private $config;
    
    
    public function execute(HTTPRequestCustom $request)
      {
        $this->init();
        
        $this->build_form();
        
        $tpl = new StringTemplate('# INCLUDE MSG # # INCLUDE FORM #');
        $tpl->add_lang($this->lang);
        $user_actual = AppContext::get_current_user();
        
        
        
        if ($this->submit_button->has_been_submited() && $this->form->validate())
          {
            // Si ce champ n'est pas vide, cela veut dire qu'on envoi la notification à un seul membre et donc pas à tous les membres.
            
            if ($this->form->get_value('username') != NULL)
              {
                // Variable qui vérifie si l'utilisateur existe grâce à la fonction plus bas get_user()
                $UserExist = $this->get_user($this->form->get_value('username'));
                
                if ($UserExist == TRUE)
                  {
                    // Si l'ID de l'utilisateur à qui on envoi une notification est différent de soit-même alors on peut envoyer la notification
                    if ($UserExist['user_id'] != $user_actual->get_id())
                      {
                        $notify = New Notifications();
                        $notify->set_sender_id($user_actual->get_id());
                        $notify->set_recipient_id($UserExist['user_id']);
                        $notify->set_reading(0);
                        $notify->set_content(NotificationsFormat::contenu('admin.msg', array(
                            'url_username' => UserUrlBuilder::profile($user_actual->get_id())->rel(),
                            'username' => $user_actual->get_display_name(),
                            'message' => $this->form->get_value('content')
                        )));
                        $notify->set_date(time());
                        Notifications::add_notification($notify);
                        
                        $tpl->put('MSG', MessageHelper::display($this->lang['admin.notifications.success.one'], MessageHelper::SUCCESS, 5));
                      }
                    // Sinon on envoi un message d'erreur
                    else
                      {
                        $tpl->put('MSG', MessageHelper::display($this->lang['admin.notifications.error.yourself'], MessageHelper::ERROR, 5));
                      }
                    
                  }
                else
                  {
                    $tpl->put('MSG', MessageHelper::display($this->lang['admin.notifications.username.404'], MessageHelper::ERROR, 5));
                  }
                
              }
            // Sinon le champ est vide donc on envoi la notification à tous les membres ...
            else
              {
                // Variable qui liste tous les id de membre existant grâce à la fonction get_all_users() disponible plus bas
                $AllUsers = $this->get_all_users();
                
                while ($row = $AllUsers->fetch())
                  {
                    $notify = New Notifications();
                    $notify->set_sender_id($user_actual->get_id());
                    $notify->set_recipient_id($row['user_id']);
                    $notify->set_reading(0);
                    $notify->set_content(NotificationsFormat::contenu('admin.msg', array(
                        'url_username' => UserUrlBuilder::profile($user_actual->get_id())->rel(),
                        'username' => $user_actual->get_display_name(),
                        'message' => $this->form->get_value('content')
                    )));
                    $notify->set_date(time());
                    Notifications::add_notification($notify);
                  }
                
                $tpl->put('MSG', MessageHelper::display($this->lang['admin.notifications.success.all'], MessageHelper::SUCCESS, 5));
              }
            
          }

        $tpl->put('FORM', $this->form->display());
        
        return new AdminNotificationsDisplayResponse($tpl, $this->lang['admin_send_notification']);
      }
    
    private function init()
      {
        $this->config            = NotificationsConfig::load();
        $this->lang              = LangLoader::get('common', 'notifications');
        $this->admin_common_lang = LangLoader::get('admin-common');
      }
    
    private function build_form()
      {
        $form     = new HTMLForm(__CLASS__);
        $fieldset = new FormFieldsetHTML('config', $this->lang['admin_send_notification']);
        $form->add_fieldset($fieldset);
        
        // Champ "Nom d'utilisateur" (S'il n'est pas rempli on envoi à tous les utilisateurs)
        $fieldset->add_field(new FormFieldTextEditor('username', $this->lang['admin.notifications.username'], '', array(
            'maxlength' => 255,
            'description' => $this->lang['admin.notifications.username.desc']
        )));
        
        // Champ "Contenu de la notiication"
        $fieldset->add_field(new FormFieldRichTextEditor('content', $this->lang['admin.notifications.content'], '', array(
            'required' => true,
            'placeholder' => $this->lang['admin.notifications.content.placeholder'],
            'description' => $this->lang['admin.notifications.content.desc']
            
        )));
        
        $this->submit_button = new FormButtonDefaultSubmit();
        $form->add_button($this->submit_button);
        $form->add_button(new FormButtonReset());
        
        $this->form = $form;
      }
    
    /**
     * [get_user Permet d'avoir les informations d'un utilisateur via son pseudo]
     * @param  [string] $username [Nom de l'utilisateur]
     * @return [array]           [Informations de l'utilisateur]
     */
    private function get_user($username)
      {
        try
          {
            $user_id = PersistenceContext::get_querier()->select_single_row(PREFIX . 'member', array(
                'user_id'
            ), 'WHERE display_name=:name', array(
                'name' => $username
            ));
          }
        catch (RowNotFoundException $e)
          {
            return false;
          }
        
        return $user_id;
      }
    
    private function get_all_users()
      {
        $user_actual = AppContext::get_current_user();
        try
          {
            $users = PersistenceContext::get_querier()->select_rows(PREFIX . 'member', array(
                'user_id'
            ), 'WHERE user_id!=:id', array(
                'id' => $user_actual->get_id()
            ));
          }
        catch (RowNotFoundException $e)
          {
            return false;
          }
        
        return $users;
      }
    
  }
?>